from os.path import join
from python_script_manager.package import PSMReader
from ..utils import (
    info_message,
    process_ok,
    process_step,
    success_message,
    hl,
    take_input
)
from .utils import (
    remove_app,
    remove_app_imports
)
from ..__main__ import main

@main.command("removeapp")
def removeapp_command():
    """Remove app in project"""
    psm = PSMReader()
    prj_name = psm.get_config().get('PROJECT_NAME', None)
    app_list = psm.get_config().get('APPS', None)
    processes = []
    process_ok(processes)

    # Taking app name to remove
    app_name = take_input("Enter name of app to remove:")

    # Removing app folder
    if not prj_name :
        raise Exception("You deleted your project name from psm.json file. Please add it.")
    if not app_name in app_list:
        raise Exception("Your app name is not in app list")
    process_step(f"Removing {hl(join(prj_name, app_name))} folder...",
                remove_app(join(prj_name, app_name)))
    processes.append(f"Removed {hl(join(prj_name, app_name))} folder")
    process_ok(processes)

    # Removing app name from psm.json
    psm_config = psm.get_config()
    psm_config['APPS'].remove(app_name)
    psm.set_config(psm_config)
    psm.write()

    # Removing app modules from __init__.py
    process_step(f"Removing imports of {hl(app_name)} from {hl(join(prj_name, '__init__.py'))}...",
                remove_app_imports(prj_name, app_name))
    processes.append(f"Removed imports of {hl(app_name)} from {hl(join(prj_name, '__init__.py'))}")
    process_ok(processes)

    # Output success message
    success_message(f"Successfully removed {hl(app_name)}")
    info_message(f"Use {hl('psm start')} to run your application")



from os.path import join
from python_script_manager.package import PSMReader
from ..utils import (
    command_process_step,
    info_message,
    process_ok,
    process_step,
    create_folder,
    success_message,
    make_compatible,
    hl
)
from .utils import (
    create_static_folders,
    create_init_py,
    create_config_py,
    create_app_py,
    write_requirements,
    create_gitignore
)
from ..__main__ import main

@main.command("startproject")
def startproject_command():
    """Create new project"""
    processes = []
    process_ok(processes)

    # Install PSM
    command_process_step(
        "Installing PSM...", "pip install --upgrade python-script-manager")
    processes.append("Installed PSM")
    process_ok(processes)

    # Create psm.json
    command_process_step("Initializing PSM...",
                        'psm init --template="flaskmng"')
    processes.append("Initialized PSM")
    process_ok(processes)

    # Create config field
    psm = PSMReader('psm.json')
    psm_config = psm.get_config()
    psm_config["APPS"] = []
    psm_config["PROJECT_NAME"] = make_compatible(psm.get_name())
    psm.set_config(psm_config)
    psm.write()

    prj_name = psm_config["PROJECT_NAME"]

    # Creating requirements.txt
    process_step("Creating requirements.txt", write_requirements)
    processes.append("Created requirements.txt")
    process_ok(processes)

    # Installing requirements.txt
    command_process_step(f"Installing {hl('requirements.txt')}...",
                        'psm install && psm freeze')
    processes.append(f"Installed {hl('requirements.txt')}")
    process_ok(processes)

    # Creating main app folder
    process_step(f"Creating {hl(prj_name)} folder...",
                create_folder(prj_name))
    processes.append(f"Created {hl(prj_name)} folder")
    process_ok(processes)

    # Creating .gitignore
    process_step(f"Creating {hl('.gitignore')}...", create_gitignore)
    processes.append(f"Created {hl('.gitignore')}")
    process_ok(processes)

    # Creating app.py
    process_step(f"Creating {hl('app.py')}...", create_app_py(prj_name))
    processes.append(f"Created {hl('app.py')}")
    process_ok(processes)

    # Creating config.py
    process_step(f"Creating {hl('config.py')}...", create_config_py)
    processes.append(f"Created {hl('config.py')}")
    process_ok(processes)

    # Creating project __init__.py
    process_step(f"Creating {hl(join(prj_name,'__init__.py'))}...",
                create_init_py(prj_name))
    processes.append(f"Created {hl(join(prj_name,'__init__.py'))}")
    process_ok(processes)

    # Creating js, css, image folders
    process_step(f"Creating {hl(join(prj_name,'static'))} folder...",
                create_static_folders(prj_name))
    processes.append(f"Created {hl(join(prj_name,'static'))} folder")
    process_ok(processes)

    # Initializing DB
    command_process_step("Initializing database...", "flask db init")
    processes.append("Initialized database")
    process_ok(processes)

    # Commiting initial migration
    command_process_step("Commiting initial migration...",
                        'flask db migrate -m "initial"')
    processes.append("Commited initial migration")
    process_ok(processes)

    # Upgrading migration
    command_process_step("Upgrading migration...", "flask db upgrade")
    processes.append("Upgraded migration")
    process_ok(processes)

    # Initializing git
    command_process_step("Initializing git...", "git init")
    processes.append("Initialized git")
    process_ok(processes)

    # Output success message
    success_message(f"Successfully created project {hl(prj_name)}")
    info_message(f"Use {hl('psm start')} to run your application")


import os
import getpass
from python_script_manager.package import PSMReader
from ..utils import (
    command_process_step,
    process_ok,
    process_step,
    success_message,
    hl,
    take_input,
    detect_venv
)
from .utils import (
    create_wsgi_py,
    create_service_file,
    create_nginx_file
)
from ..__main__ import main

@main.command("deployproject")
def deployproject_command():
    psm = PSMReader()
    prj_name = psm.get_config().get('PROJECT_NAME', None)
    processes = []
    process_ok(processes)

    # Creating wsgi.py
    process_step(f"Creating {hl('wsgi.py')}...", create_wsgi_py(prj_name))
    processes.append(f"Created {hl('wsgi.py')}")
    process_ok(processes)

    # Creating service file
    venv_name = detect_venv()
    project_path = os.getcwd()
    username = getpass.getuser()
    process_step(f"Creating {hl(f'/etc/systemd/system/{prj_name}.service')}...",
                 create_service_file(project_path, prj_name, username, venv_name))
    processes.append(
        f"Created {hl(f'/etc/systemd/system/{prj_name}.service')}")
    process_ok(processes)

    # Starting project
    command_process_step("Starting project...", f"systemctl start {prj_name}")
    processes.append("Started project")
    process_ok(processes)

    # Enabling project
    command_process_step("Enabling project...", f"systemctl enable {prj_name}")
    processes.append("Enabled project")
    process_ok(processes)

    # Taking domain name as input
    domain_name = take_input("Enter your domain name(without any subdomain): ")

    # Creating nginx file
    process_step(f"Creating {hl(f'/etc/nginx/sites-available/{prj_name}')}...",
                 create_nginx_file(project_path, prj_name, domain_name))
    processes.append(
        f"Created {hl(f'/etc/nginx/sites-available/{prj_name}')}")
    process_ok(processes)

    # Link nginx file
    command_process_step("Linking nginx file...", f"ln -s /etc/nginx/sites-available/{prj_name}.conf /etc/nginx/sites-enabled")
    processes.append("Linked nginx file")
    process_ok(processes)

    # Restart nginx
    command_process_step("Restarting nginx...", "systemctl restart nginx")
    processes.append("Restarted nginx")
    process_ok(processes)

    # Allowing Nginx
    command_process_step("Allowing nginx...", "ufw allow 'Nginx Full'")
    processes.append("Allowed nginx")
    process_ok(processes)

    # Show success message
    success_message(f"Successfully deployed {hl(prj_name)}")


import shutil
from os.path import join

def remove_app(app_folder):
    def wrapper():
        shutil.rmtree(app_folder)
    return wrapper

def remove_app_imports(project_folder,app_name):
    def wrapper():
        result = ""
        with open(join(project_folder,"__init__.py")) as f:
            code = f.read()
            flag = True
            for line in code.split('\n'):
                if f"#[APP] {app_name}" in line:
                    flag = False
                    continue
                elif f"#[/APP] {app_name}" in line:
                    flag = True
                    continue
                elif not flag:
                    continue
                else:
                    result+=line+"\n"
        with open(join(project_folder,"__init__.py"),'w') as f:
            f.write(result)
    return wrapper


from os.path import join

def create_app_init_py(app_name):
    def wrapper():
        with open(join(app_name, "__init__.py"), "w") as f:
            f.write("")
    return wrapper

def create_models_py(app_name):
    def wrapper():
        with open(join(app_name,"models.py"),"w") as f:
            body_text = """\
from .. import db
            """
            f.write(body_text)
    return wrapper

def create_forms_py(app_name):
    def wrapper():
        with open(join(app_name,"forms.py"),"w") as f:
            body_text = """\
from flask_wtf import FlaskForm
            """
            f.write(body_text)
    return wrapper

def create_routes_py(prj_name,app_name):
    def wrapper():
        with open(join(prj_name,app_name,"routes.py"),"w") as f:
            body_text = f"""\
from flask import render_template, redirect, url_for, Blueprint

{app_name} = Blueprint('{app_name}', __name__, template_folder='templates')

@{app_name}.route('/')
def index():
    return "Hello!!!"
            """
            f.write(body_text)
    return wrapper

def append_app_datas(prj_name, app_name):
    def wrapper():
        with open(join(prj_name,"__init__.py"), "a") as f:
            body_text = f"""
#[APP] {app_name}
from .{app_name}.models import *
from .{app_name}.routes import {app_name}
app.register_blueprint({app_name})
#[/APP] {app_name}
            """
            f. write(body_text)
    return wrapper


import setuptools
from python_script_manager.package import PSMReader

with open("README.md","r") as fh:
    long_description = fh.read()

psm = PSMReader('psm.json')

def parse_requirements(filename):
    """ load requirements from a pip requirements file """
    lineiter = (line.strip() for line in open(filename))
    return [line for line in lineiter if line and not line.startswith("#")]
reqs = parse_requirements('requirements.txt')

setuptools.setup(
    name=psm.get_name(),
    version=psm.get_version(),
    author=psm.get_author(),
    author_email=psm.get_author_email(),
    description=psm.get_description(),
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=psm.get_url(),
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Software Development",
        "Natural Language :: English"
    ],
    install_requires=reqs,
    setup_requires=reqs,
    python_requires='>=3.6',
    entry_points='''
        [console_scripts]
        flaskmng=flaskmng.__main__:command_line_interface
    ''',
)